class Allocator
{
    // This is allocator class which will store all statistics in the mem_chunks_.
    //Deallocation happens when the object is destroyed.
    std::list <std::unique_ptr <unsigned char []> > mem_chunks_;
    const size_t chunk_size_;
  public:
    Allocator (size_t s) :chunk_size_(s) {}
    static int pre_alloc(void *p, void **u) {
      return (static_cast <Allocator *> (p))->alloc(u);
    }
    int alloc(void **u) {
      std::unique_ptr <unsigned char []> chunk(new unsigned char [chunk_size_]);
      memset(&(chunk[0]), 0, chunk_size_); //scTDC does not 'zeroing' memory supplied!
      *u = &(chunk[0]);
      mem_chunks_.push_back(std::move(chunk));
      return 0;
    }
};
int statistics_pipe_ex()
{
  int dd = sc_tdc_init_inifile("tdc_gpx3.ini"); //init the hardware and get device descriptor
  if (dd < 0) {
    char error_description[ERRSTRLEN];
    sc_get_err_msg(dd, error_description);
    printf("error! code: %d, message: %s\n", dd, error_description);
    return dd;
  }
  sc_pipe_statistics_params_t prms;
  memset(&prms, 0, sizeof(prms));
  Allocator mem(sizeof(statistics_t));
  prms.allocator_owner = static_cast <void *> (&mem);
  prms.allocator_cb = &(mem.pre_alloc);
  int pd = sc_pipe_open2(dd, STATISTICS, (void *)&prms);
  if (pd < 0) {
    char error_description[ERRSTRLEN];
    sc_get_err_msg(pd, error_description);
    printf("error! code: %d, message: %s\n", pd, error_description);
    sc_tdc_deinit(dd);
    return pd;
  }
  int ret = sc_tdc_start_measure2(dd, 200); // start 200 ms measure
  if (ret < 0) {
    char error_description[ERRSTRLEN];
    sc_get_err_msg(ret, error_description);
    printf("error! code: %d, message: %s\n", ret, error_description);
    sc_pipe_close2(dd, pd);
    sc_tdc_deinit(dd);
    return ret;
  }
  statistics_t *stat;
  //During measure processing engine will call mem.pre_alloc function, which
  //allocates 1k chunk of memory, save it in the list and return to the processor.
  //The next function will unblock after measure is finished. Application
  //will get back statistics in the memory, which will be deallocated when mem
  //object is destroyed. If application call sc_start_measure2 & sc_pipe_read2()
  //many times here mem object will accumulate memory chunks in the list.
  //Other nice application of this feature is making accumulation of the data
  // in the same memory space by giving always the same pointer. scTDC will not
  //'zeroed' memory supplied and data will accumulate in the same place.
  //This works for images and histograms as well.
  int ret = sc_pipe_read2(dd, pd, (void *)&stat, UINT32_MAX);
  if (ret < 0) {
    char error_description[ERRSTRLEN];
    sc_get_err_msg(ret, error_description);
    printf("error! code: %d, message: %s\n", ret, error_description);
    sc_pipe_close2(dd, pd);
    sc_tdc_deinit(dd);
    return ret;
  }
  //Here we can do something with statistics
  sc_pipe_close2(dd, pd);
  sc_tdc_deinit2(dd);
  //Here statistics is still accessible because mem object is still on the stack.
  //If 'internal' memory allocation used this would be not true.
  return 0;
}
